<?php

	ini_set('max_execution_time', 300);

	if ($_SERVER['REQUEST_METHOD'] != 'POST')
	{
		header('Location: index.html');
	}
	
	// setting global variables
	define("IP_FIRST_AFTER_MAXCGC", 1);
	define("IP_ALIGNED_WITH_MINGC", 2);
	$windowSizePercentage = 1.0;
	$predictionIntervalSize = 1;
	$gcKnockout = 10;
	$lengthKnockout = 2000;
	
	if(!file_exists($_FILES['sequenceInput']['tmp_name']) || !is_uploaded_file($_FILES['sequenceInput']['tmp_name'])) {
		echo '[1003] Error uploading file';
		exit;
	}
		
	if(ISSET($_POST["windowSize"]) && $_POST["windowSize"] != 1 && $_POST["windowSize"] >= 0.1 && $_POST["windowSize"] <= 5) {
		$windowSizePercentage = round($_POST["windowSize"], 1);
	}

	if(ISSET($_POST["predictionIntervalSize"]) && ($_POST["predictionIntervalSize"] == 1 || $_POST["predictionIntervalSize"] == 2)) {
		$predictionIntervalSize = $_POST["predictionIntervalSize"];
	}
	
	if(ISSET($_POST["gcKnockout"]) && $_POST["gcKnockout"] > 0 && $_POST["gcKnockout"] < 100) {
		$gcKnockout = $_POST["gcKnockout"];
	}
	
	if(ISSET($_POST["lengthKnockout"]) && $_POST["lengthKnockout"] > 0 && $_POST["lengthKnockout"] < 100000) {
		$lengthKnockout = $_POST["lengthKnockout"];
	}
	
	if(ISSET($_POST["minGCContentThreshold"]) && $_POST["minGCContentThreshold"] > 10 && $_POST["minGCContentThreshold"] < 100) {
		$minGCContentThreshold = $_POST["minGCContentThreshold"];
	}
	
	if(ISSET($_POST["entropyThreshold"]) && $_POST["entropyThreshold"] >= 1 && $_POST["entropyThreshold"] <= 5) {
		$entropyThreshold = $_POST["entropyThreshold"];
	}

	if(ISSET($_POST["gcSkewThreshold"]) && $_POST["gcSkewThreshold"] >= 0.05 && $_POST["gcSkewThreshold"] <= 1) {
		$gcSkewThreshold = $_POST["gcSkewThreshold"];
	}
	
	if(ISSET($_POST["varianceThreshold"]) && $_POST["varianceThreshold"] >= 1 && $_POST["varianceThreshold"] <= 1000000000000) {
		$varianceThreshold = $_POST["varianceThreshold"];
	}

	//loading library with calculations
	require_once("_APIengine.php");
	
	$start = microtime(true);
	
	$genome = new Genome();
	
	$knockoutResults = $genome->CheckKnockoutCriteria();
	if (!$knockoutResults["isValid"])
	{
		$link = sprintf("noresult.php?ko=1&gc=%d&l=%d", $knockoutResults["gcContent"], $knockoutResults["length"]);
		header("Location: $link");
	}
	
	$validationResults = $genome->GlobalCheck();

	if (!$validationResults["isValid"])
	{
		$link = sprintf("noresult.php?val=1&gc=%f&e=%f&gcs=%f&v=%d", $validationResults["gcContent"], $validationResults["entropy"], $validationResults["gcSkew"], $validationResults["variance"]);
		header("Location: $link");
	}
	
	$skews["CALCULATIONS"] = $genome->CalculateSkews($windowSizePercentage);
	$skews["SUMMARY"] = $skews["CALCULATIONS"]["summary"];
	$skews["WSIZE_PERCENTAGE"] = $skews["CALCULATIONS"]["windowSizePercentage"];
	$skews["WSIZE"] = $skews["CALCULATIONS"]["windowSize"];
	$data["skews"] = $skews;
	$ipAfterMaxCGC = $genome->getSpecificInflectionPoint(IP_FIRST_AFTER_MAXCGC);
	$ipAfterMinGC = $genome->getSpecificInflectionPoint(IP_ALIGNED_WITH_MINGC);
	
	/*
	Possible values:
	P - oriC is at beggining/end of the genome. One of three conditions must be met:
		(1) There is no IP after CGC max
		(2) X-axis overlap between values of CGC's IP (which is after CGC max) and GC min is within two last windows
		(3) CGC max is within first window
	S - oriC is within the genome
	*/
	
	$predictionPositionCode = (($data["skews"]["SUMMARY"]["maxcGC"]["positionLeft"] < $data["skews"]["WSIZE"]) || !ISSET($ipAfterMaxCGC) || ((ISSET($ipAfterMinGC)) ? ($ipAfterMinGC["positionRight"] > (LENGTH - 2*$data["skews"]["WSIZE"] - 1) ? true : false) : false)) ? "P" : "S" ;
	
	
	$predictionIntervalSizeInfo = "one-window size length";
	if ($predictionIntervalSize == 2)
	{
		$predictionIntervalSizeInfo = "two-window sizes length";
	}
	if ($predictionPositionCode == "S")
	{
		$predictionWindowLeft = round($ipAfterMinGC["positionLeft"] / $skews["WSIZE"]);
		$predictionWindowRight = $predictionWindowLeft + $predictionIntervalSize;
		$resultSentence = sprintf("within the window starting at %s bp", number_format($ipAfterMinGC["positionLeft"]));
		$predictionInterval = sprintf("%s..%s", number_format($ipAfterMinGC["positionLeft"]), number_format($ipAfterMinGC["positionLeft"]+$predictionIntervalSize*$skews["WSIZE"]));
	}
	else	  // if predictionPositionCode is P
	{
		$predictionWindowLeft = 0;
		$predictionWindowRight = $predictionWindowLeft + $predictionIntervalSize;
		$resultSentence = "at the beggining of the genome";
		$predictionInterval = sprintf("0..%s", number_format($predictionIntervalSize*$skews["WSIZE"]));
	}
	$time_elapsed_secs = microtime(true) - $start;

?>

<html>
  <head>
  	<title>ForiC - Fast oriC predictor</title>
    <link href="css/bootstrap.min.css" rel="stylesheet" crossorigin="anonymous">
	<script src="js/echarts.min.js"></script>
    <script src="js/echarts-theme.js"></script>
	<link rel="icon" type="image/x-icon" href="img/favicon-16x16.png?v=2">
  </head>
  <body>
	<div class="container">
			  <nav class="navbar navbar-expand-lg navbar-light bg-light">
				<div class="container">
				  <a class="navbar-brand" href="#">ForiC</a>
				  <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav" aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
					<span class="navbar-toggler-icon"></span>
				  </button>
				  <div class="collapse navbar-collapse" id="navbarNav">
					<ul class="navbar-nav ms-auto">
					  <li class="nav-item">
						<a class="nav-link active" href="index.html">Home</a>
					  </li>
					  <li class="nav-item">
						<a class="nav-link" href="sampleData.html">Sample Data</a>
					  </li>
					  <li class="nav-item">
						<a class="nav-link" href="sampleResult.html">Sample Output</a>
					  </li>
					  <li class="nav-item">
						<a class="nav-link" href="about.html">About</a>
					  </li>
					</ul>
				  </div>
				</div>
			  </nav>
			<h1><?php printf("Accession %s", ACCESSION) ?></h1>
			<p><p class='lead mb-4'>ForiC predicts oriC position <?php echo $resultSentence; ?> (marked with green vertical line). Prediction details are shown below.</p></p>
			<div style="width:100%; height:80%">
				<div id="main" style="width: 100%; height: 100%">
					<script type="text/javascript">
					  var myChart = echarts.init(document.getElementById("main"), "ForiC");

					  // X-axis values
					  var xAxisData = <?php echo $genome->GetWindowValues(); ?>

					  // Y-axis values for each line
					  var atData = <?php echo $genome->GetATvaluesForGraph(); echo ";";?>
					  
					  var gcData = <?php echo $genome->GetGCvaluesForGraph(); echo ";";?>
					  
					  var cgcData = <?php echo $genome->GetCGCvaluesForGraph(); echo ";";?>
					  
					  var catData = <?php echo $genome->GetCATvaluesForGraph(); echo ";";?>
					  
					  var cgcIPData = <?php echo $genome->GetInflectionPointsValuesForGraph(); echo ";";?>

					  var catAdditionalPoints = generateCatAdditionalPoints(
						cgcIPData,
						cgcData,
						xAxisData
					  );

					  console.log(catAdditionalPoints);

					  var option = {
						title: {
						  text: "ForiC Graphic Representation",
						  left: "center",
						},
						tooltip: {
						  trigger: "axis",
						},
						legend: {
						  data: ["AT", "GC", "CGC", "CAT"],
						  top: "bottom",
						},
						toolbox: {
						  show: true,
						  feature: {
							dataZoom: {
							  yAxisIndex: "none",
							},
							dataView: { readOnly: false },
							magicType: { type: ["line", "bar"] },
							restore: {},
							saveAsImage: {},
						  },
						},
						xAxis: {
						  type: "category",
						  data: xAxisData,
						},
						yAxis: {
						  type: "value",
						},
						series: [
						  {
							name: "AT",
							type: "line",
							data: atData,
							sampling: "lttb",
							smooth: true,
							showSymbol: false,
							markPoint: {
							  data: [
								{ type: "max", name: "Max", label: { formatter: "Max" } },
								{ type: "min", name: "Min", label: { formatter: "Min" } },
							  ],
							},
						  },
						  {
							name: "GC",
							type: "line",
							data: gcData,
							sampling: "lttb",
							smooth: true,
							showSymbol: false,
							markPoint: {
							  data: [
								{ type: "max", name: "Max", label: { formatter: "Max" } },
								{ type: "min", name: "Min", label: { formatter: "Min" } },
							  ],
							},
													
							markLine: {
							  //xAxis is index here from xAxisData
							  data: [
									{ 
										xAxis: <?php echo ($predictionWindowLeft); ?>, 
										lineStyle: {color: 'black', width: 2 }, 
										label: { formatter: "START", position: 'insideEndTop' }
									},  
									{ 	xAxis: <?php echo ($predictionWindowRight); ?>, 
										lineStyle: {color: 'green',  width: 2 }, 
										label: { formatter: "END"}
									}],
							},
						  },
						  {
							name: "CGC",
							type: "line",
							data: cgcData,
							sampling: "lttb",
							smooth: true,
							showSymbol: false,
							markPoint: {
							  data: [
								...catAdditionalPoints,
								{ type: "max", name: "Max", label: { formatter: "Max" } },
								{ type: "min", name: "Min", label: { formatter: "Min" } }
							  ],
							},
						  },
						  {
							name: "CAT",
							type: "line",
							data: catData,
							showSymbol: false,
							smooth: true,
							markPoint: {
							  data: [{ type: "max", name: "Max", label: { formatter: "Max" } },
								{ type: "min", name: "Min", label: { formatter: "Min" } }],
							},
						  },
						],
					  };

					  function generateCatAdditionalPoints(yValues, cgcData) {
						let points = [];
						for (let i = 0; i < yValues.length; i++) {
						  let index = cgcData.indexOf(yValues[i]);
						  if (index !== -1) {
							points.push({
							  xAxis: index,
							  yAxis: yValues[i],
							  symbol: "circle",
							  symbolSize: 10,
							});
						  }
						}
						return points;
					  }
					  myChart.setOption(option);
					</script>
				</div>
				<p style="text-align:center; font-size:small; margin-bottom:5px;">Each graph shows global maximum and minimum values, which are clearly indicated. Dots on the CGC skew represent inflection points. <br>The black vertical line indicates the start of the window containing the predicted oriC position, while the green vertical line denotes its end.</p>
				<div class="row">
					<div class="col">
					</div>
					<div class="col-8">
					<table class="table caption-top">
						<caption>ForiC calculations</caption>
						<thead>
							<tr>
								<th scope="col">Attribute</th>
								<th scope="col">Value</th>
							</tr>
						</thead>
						<tbody>
							<tr>
								<th scope="row">Genome length</th>
								<td><?php echo number_format(LENGTH) . " bp"; ?></td>
							</tr>
							<tr>
								<th scope="row">Window size</th>
								<td><?php printf("%s bp (%.1f%% of genome length)", number_format($skews["WSIZE"]), $windowSizePercentage); ?></td>
							</tr>
							<tr>
								<th scope="row">Number of windows</th>
								<td><?php echo $genome->GetNumberOfWindows(); ?></td>
							</tr>
							<tr>
								<th scope="row">ForiC execution</th>
								<td><?php echo number_format($time_elapsed_secs, 2) . " seconds";?></td>
							</tr>
							<tr>
								<th scope="row">Prediction interval</th>
								<td><?php printf("%s (%s)", $predictionInterval, $predictionIntervalSizeInfo); ?></td>
							</tr>
							<tr>
								<th scope="row">CGC global maximum position</th>
								<td><?php printf("%s..%s", number_format($skews["SUMMARY"]["maxcGC"]["positionLeft"]), number_format($skews["SUMMARY"]["maxcGC"]["positionRight"]));  ?></td>
							</tr>
							<tr>
								<th scope="row">Is there an inflection point after CGC global maximum?</th>
								<td><?php echo ISSET($ipAfterMaxCGC) ? "Yes" : "No" ?></td>
							</tr>	
							<tr>
								<th scope="row">Is there a x-axis overlap between an inflection point (after CGC global maximum) and GC global minumum?</th>
								<td><?php echo ISSET($ipAfterMinGC) ? "Yes" : "No" ?></td>
							</tr>	
							<tr>
								<th scope="row">GC content</th>
								<td><?php echo number_format($validationResults["gcContent"], 2). "%"; ?></td>
							</tr>	
							<tr>
								<th scope="row">Entropy</th>
								<td><?php echo number_format($validationResults["entropy"], 6); ?></td>
							</tr>	
							<tr>
								<th scope="row">GC skew</th>
								<td><?php echo number_format($validationResults["gcSkew"], 6); ?></td>
							</tr>	
							<tr>
								<th scope="row">Variance</th>
								<td><?php echo $validationResults["variance"]; ?></td>
							</tr>	
						</tbody>
					</table>
					</div>
					<div class="col">
					</div>
				</div>
			</div>

	
		</div>
		<div class="b-example-divider"></div>
	



	
  </body>
</html>
